import pygame
from io import BytesIO
import requests


class board:
    def __init__(self, N):
        self.N = N
        self.colors = [(255,255,255), (255,153,204)]
        self.surface_sz = 640
        self.grid_sz = self.surface_sz // self.N
        self.surface_sz = self.N * self.grid_sz
        self.surface = None
        self.queen = self.read_queen_image()
        self.queen = pygame.transform.scale(self.queen, (self.grid_sz, self.grid_sz))
        self.offset = (self.grid_sz-self.queen.get_width()) // 2

        self.create_board()
    
    def read_queen_image(self):
        url = 'https://github.com/jhan15/nqviz/blob/master/nqviz/queen.png?raw=true'
        r = requests.get(url)
        dataBytesIO = BytesIO(r.content)
        queen_img = pygame.image.load(dataBytesIO)
        return queen_img
    
    def create_board(self):
        pygame.init()
        title = str(self.N) + '-queen chessboard'
        pygame.display.set_caption(title)
        self.surface = pygame.display.set_mode((self.surface_sz, self.surface_sz))

        for row in range(self.N):
            color_ind = row % 2
            for col in range(self.N):
                grid = (col*self.grid_sz, row*self.grid_sz, self.grid_sz, self.grid_sz)
                self.surface.fill(self.colors[color_ind], grid)
                color_ind = (color_ind + 1) % 2
        
        pygame.display.flip()
    
    def place_queen(self, pos):
        x1 = pos[0] * self.grid_sz
        y1 = pos[1] * self.grid_sz
        x2 = (pos[0] + 1) * self.grid_sz
        y2 = (pos[1] + 1) * self.grid_sz

        self.surface.blit(self.queen, (x1+self.offset,y1+self.offset))
        pygame.display.update(x1,y1,x2,y2)
        pygame.event.pump()
        pygame.time.delay(500)
    
    def remove_queen(self, pos):
        x1 = pos[0] * self.grid_sz
        y1 = pos[1] * self.grid_sz
        x2 = (pos[0] + 1) * self.grid_sz
        y2 = (pos[1] + 1) * self.grid_sz
        color_ind = (pos[0] + pos[1]) % 2

        grid = (x1, y1, self.grid_sz, self.grid_sz)
        self.surface.fill(self.colors[color_ind], grid)
        pygame.display.update(x1,y1,x2,y2)
        pygame.event.pump()
        pygame.time.delay(500)

    def quit(self):
        pygame.quit()



import pygame
from random import shuffle
from .board import board


class Nqueen:
    def __init__(self, N):
        self.N = N
        self.cb = board(self.N)
        self.num_cb = [[0] * self.N for _ in range(self.N)]
        self.finished = False
        self.start_procedure()
    
    def start_procedure(self):
        while True:
            ev = pygame.event.poll()
            if ev.type == pygame.QUIT:
                break
            
            if not self.finished:
                if self.backtracking():
                    self.finished = True
    
        self.cb.quit()

    def backtracking(self):
        cols = [i for i in range(self.N)]
        shuffle(cols)

        if self.solver(cols, 0) == False:
            print('Solution does not exist!')
            return False

        return True

    def solver(self, cols, col):
        if col == self.N:
            return True
        
        rows = [i for i in range(self.N)]
        shuffle(rows)

        for row in rows:
            if self.is_safe(row, cols[col]):
                self.num_cb[row][cols[col]] = 1
                self.cb.place_queen((cols[col], row))

                if self.solver(cols, col+1) == True:
                    return True
                
                self.num_cb[row][cols[col]] = 0
                self.cb.remove_queen((cols[col], row))
        
        return False

    def is_safe(self, row, col):
        for i in range(self.N):
            if self.num_cb[row][i] == 1:
                return False
        for i, j in zip(range(row, -1, -1), range(col, -1, -1)):
            if self.num_cb[i][j] == 1:
                return False
        for i, j in zip(range(row, self.N, 1), range(col, -1, -1)):
            if self.num_cb[i][j] == 1:
                return False
        for i, j in zip(range(row, -1, -1), range(col, self.N, 1)):
            if self.num_cb[i][j] == 1:
                return False
        for i, j in zip(range(row, self.N, 1), range(col, self.N, 1)):
            if self.num_cb[i][j] == 1:
                return False
    
        return True

if __name__ == "__main__":
    nq = Nqueen(6)
    


from setuptools import setup
from os import path

# read the contents of README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='nqviz',
      version='0.0.6',
      description='N Queen Visualizer',
      url='https://github.com/jhan15/nqviz',
      author = 'Jianming Han',
      author_email = 'hjma810@126.com',
      license='MIT',
      classifiers=[
          'Development Status :: 4 - Beta',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.8',
      ],
      packages=['nqviz'],
      install_requires=['pygame>=2.0.1'],
      python_requires='>=3',
      long_description=long_description,
      long_description_content_type='text/markdown',
      zip_safe=False)
